import 'dart:convert';
import 'dart:typed_data';

import 'package:binarize/binarize.dart';

class _String extends PayloadType<String> {
  const _String();

  @override
  int length(String value) {
    final encoded = utf8.encode(value);
    return encoded.length + 4;
  }

  @override
  String get(ByteData data, int offset) {
    final length = data.getUint32(offset);
    final byteOffset = offset + 4;
    final bytes = <int>[];

    for (var i = byteOffset; i < byteOffset + length; i++) {
      bytes.add(data.getUint8(i));
    }

    return utf8.decode(bytes);
  }

  @override
  void set(String value, ByteData data, int offset) {
    final bytes = utf8.encode(value);

    data.setUint32(offset, bytes.length);
    var counter = offset + 4;

    for (final byte in bytes) {
      data.setUint8(counter++, byte);
    }
  }
}

/// Converts a string into an encoded UTF-8 list.
///
/// The value can not be longer than 2<sup>32</sup> - 1.
///
/// It has a byte length of `4 + len(value)`.
///
/// Usage:
///
/// ```dart
/// payload.set(string, 'Hello world');
/// ```
const string = _String();
